@@ -97,7 +97,7 @@ function wait_cdi_available {
 
 function configure_uploadproxy_override {
   host_port=$(./cluster-up/cli.sh ports uploadproxy | xargs)
-  override="127.0.0.1:$host_port"
+  override="https://127.0.0.1:$host_port"
   _kubectl patch cdi ${CR_NAME} --type=merge -p '{"spec": {"config": {"uploadProxyURLOverride": "'"$override"'"}}}'
 }
 
@@ -275,12 +275,6 @@ if [ "${KUBEVIRT_PROVIDER}" != "external" ]; then
   configure_uploadproxy_override
   # Tell prometheus to watch our namespace
   configure_prometheus
-  if [ "$KUBEVIRT_STORAGE" == "nfs" ]; then
-  # nfs writing started to consistently breach the dirty_ratio, causing OOMKills
-  # we think the problem sits somewhere around the fsync calls to writeout to nfs being slow/failing
-  # https://github.com/kubevirt/containerized-data-importer/pull/3023#issuecomment-1913529241
-  _kubectl patch cdi ${CR_NAME} --type merge -p '{"spec":{"config":{"podResourceRequirements": {"limits": {"cpu": "750m", "memory": "1Gi"}, "requests": {"cpu": "100m", "memory": "60M"}}}}}'
-  fi
 fi
 
 # Grab all the CDI crds so we can check if they are structural schemas
