@@ -24,7 +24,9 @@ import (
 	"encoding/json"
 	"fmt"
 
+	"github.com/docker/go-units"
 	snapclient "github.com/kubernetes-csi/external-snapshotter/client/v6/clientset/versioned"
+
 	admissionv1 "k8s.io/api/admission/v1"
 	v1 "k8s.io/api/core/v1"
 	apiequality "k8s.io/apimachinery/pkg/api/equality"
@@ -34,6 +36,7 @@ import (
 	k8sfield "k8s.io/apimachinery/pkg/util/validation/field"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/klog/v2"
+
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
@@ -215,12 +218,6 @@ func (wh *dataVolumeValidatingWebhook) validateDataVolumeSpec(request *admission
 			})
 			return causes
 		}
-		if request.Operation == admissionv1.Create {
-			cause := wh.validateDataVolumeSourcePVC(spec.Source.PVC, field.Child("source", "PVC"), spec)
-			if cause != nil {
-				causes = append(causes, *cause)
-			}
-		}
 	}
 	if spec.Source.Snapshot != nil {
 		if spec.Source.Snapshot.Namespace == "" || spec.Source.Snapshot.Name == "" {
@@ -231,12 +228,6 @@ func (wh *dataVolumeValidatingWebhook) validateDataVolumeSpec(request *admission
 			})
 			return causes
 		}
-		if request.Operation == admissionv1.Create {
-			cause := wh.validateDataVolumeSourceSnapshot(spec.Source.Snapshot, field.Child("source", "Snapshot"), spec)
-			if cause != nil {
-				causes = append(causes, *cause)
-			}
-		}
 	}
 
 	return causes
@@ -264,61 +255,6 @@ func (wh *dataVolumeValidatingWebhook) validateSourceRef(request *admissionv1.Ad
 			Field:   field.Child("sourceRef", "Name").String(),
 		}
 	}
-	if request.Operation != admissionv1.Create {
-		return nil
-	}
-	ns := namespace
-	if spec.SourceRef.Namespace != nil && *spec.SourceRef.Namespace != "" {
-		ns = spec.SourceRef.Namespace
-	}
-	dataSource, err := wh.cdiClient.CdiV1beta1().DataSources(*ns).Get(context.TODO(), spec.SourceRef.Name, metav1.GetOptions{})
-	if err != nil {
-		if k8serrors.IsNotFound(err) {
-			return &metav1.StatusCause{
-				Type:    metav1.CauseTypeFieldValueNotFound,
-				Message: fmt.Sprintf("SourceRef %s/%s/%s not found", spec.SourceRef.Kind, *ns, spec.SourceRef.Name),
-				Field:   field.Child("sourceRef").String(),
-			}
-		}
-		return &metav1.StatusCause{
-			Message: err.Error(),
-			Field:   field.Child("sourceRef").String(),
-		}
-	}
-	switch {
-	case dataSource.Spec.Source.PVC != nil:
-		return wh.validateDataVolumeSourcePVC(dataSource.Spec.Source.PVC, field.Child("sourceRef"), spec)
-	case dataSource.Spec.Source.Snapshot != nil:
-		return wh.validateDataVolumeSourceSnapshot(dataSource.Spec.Source.Snapshot, field.Child("sourceRef"), spec)
-	}
-
-	return &metav1.StatusCause{
-		Message: fmt.Sprintf("Empty source field in '%s'. DataSource may not be ready yet", dataSource.Name),
-		Field:   field.Child("sourceRef").String(),
-	}
-}
-
-func (wh *dataVolumeValidatingWebhook) validateDataVolumeSourcePVC(PVC *cdiv1.DataVolumeSourcePVC, field *k8sfield.Path, spec *cdiv1.DataVolumeSpec) *metav1.StatusCause {
-	sourcePVC, err := wh.k8sClient.CoreV1().PersistentVolumeClaims(PVC.Namespace).Get(context.TODO(), PVC.Name, metav1.GetOptions{})
-	if err != nil {
-		// We allow the creation of a clone DV even if the source PVC doesn't exist.
-		// The validation will be completed once the source PVC is created.
-		if k8serrors.IsNotFound(err) {
-			return nil
-		}
-		return &metav1.StatusCause{
-			Message: err.Error(),
-			Field:   field.String(),
-		}
-	}
-
-	if err := cc.ValidateClone(sourcePVC, spec); err != nil {
-		return &metav1.StatusCause{
-			Type:    metav1.CauseTypeFieldValueInvalid,
-			Message: err.Error(),
-			Field:   field.String(),
-		}
-	}
 
 	return nil
 }
@@ -397,29 +333,6 @@ func validateDataSourceRef(dataSource *v1.TypedObjectReference, field *k8sfield.
 	return causes
 }
 
-func (wh *dataVolumeValidatingWebhook) validateDataVolumeSourceSnapshot(snapshot *cdiv1.DataVolumeSourceSnapshot, field *k8sfield.Path, spec *cdiv1.DataVolumeSpec) *metav1.StatusCause {
-	sourceSnapshot, err := wh.snapClient.SnapshotV1().VolumeSnapshots(snapshot.Namespace).Get(context.TODO(), snapshot.Name, metav1.GetOptions{})
-	if err != nil {
-		if k8serrors.IsNotFound(err) {
-			return nil
-		}
-		return &metav1.StatusCause{
-			Message: err.Error(),
-			Field:   field.String(),
-		}
-	}
-
-	if err := cc.ValidateSnapshotClone(sourceSnapshot, spec); err != nil {
-		return &metav1.StatusCause{
-			Type:    metav1.CauseTypeFieldValueInvalid,
-			Message: err.Error(),
-			Field:   field.String(),
-		}
-	}
-
-	return nil
-}
-
 func validateStorageClassName(spec *cdiv1.DataVolumeSpec, field *k8sfield.Path) *metav1.StatusCause {
 	var sc *string
 
@@ -438,7 +351,7 @@ func validateStorageClassName(spec *cdiv1.DataVolumeSpec, field *k8sfield.Path)
 
 func validateStorageSize(spec *cdiv1.DataVolumeSpec, field *k8sfield.Path) (*metav1.StatusCause, bool) {
 	var name string
-	var resources v1.ResourceRequirements
+	var resources v1.VolumeResourceRequirements
 
 	if spec.PVC != nil {
 		resources = spec.PVC.Resources
@@ -461,6 +374,16 @@ func validateStorageSize(spec *cdiv1.DataVolumeSpec, field *k8sfield.Path) (*met
 			}
 			return &cause, false
 		}
+		if pvcSize.Value() < units.MiB {
+			// Kubevirt doesn't allow disks smaller than 1MiB.
+			// Rejecting DataVolume for consistency.
+			cause := metav1.StatusCause{
+				Type:    metav1.CauseTypeFieldValueInvalid,
+				Message: fmt.Sprintf("%s size should be at least 1MiB", name),
+				Field:   field.Child(name, "resources", "requests", "size").String(),
+			}
+			return &cause, false
+		}
 	} else if spec.Storage == nil || !isClone {
 		cause := metav1.StatusCause{
 			Type:    metav1.CauseTypeFieldValueInvalid,
