@@ -23,6 +23,7 @@ import (
 	. "github.com/onsi/gomega"
 
 	snapshotv1 "github.com/kubernetes-csi/external-snapshotter/client/v6/apis/volumesnapshot/v1"
+
 	corev1 "k8s.io/api/core/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
@@ -32,6 +33,7 @@ import (
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
 	"k8s.io/utils/ptr"
+
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
@@ -84,7 +86,7 @@ var _ = Describe("SnapshotClonePhase test", func() {
 			},
 			Spec: corev1.PersistentVolumeClaimSpec{
 				StorageClassName: &storageClassName,
-				Resources: corev1.ResourceRequirements{
+				Resources: corev1.VolumeResourceRequirements{
 					Requests: corev1.ResourceList{
 						corev1.ResourceStorage: resource.MustParse("20Gi"),
 					},
@@ -158,7 +160,7 @@ var _ = Describe("SnapshotClonePhase test", func() {
 			assertNotFound(err)
 		})
 
-		It("should should create the claim when ready", func() {
+		It("should create the claim when ready", func() {
 			snapshot := getSnapshot()
 			snapshot.Status.ReadyToUse = ptr.To[bool](true)
 
