@@ -19,9 +19,9 @@ package populators
 import (
 	"context"
 	"reflect"
-	"regexp"
 
 	"github.com/go-logr/logr"
+
 	corev1 "k8s.io/api/core/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -29,6 +29,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/record"
+
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
@@ -61,10 +62,6 @@ const (
 	uidField = "metadata.uid"
 )
 
-var (
-	validLabelsMatch = regexp.MustCompile(`^([\w.]+\.kubevirt.io|kubevirt.io)/\w+$`)
-)
-
 // Interface to store populator-specific methods
 type populatorController interface {
 	// Returns the specific populator CR
@@ -119,9 +116,8 @@ func CreateCommonPopulatorIndexes(mgr manager.Manager) error {
 
 func addCommonPopulatorsWatches(mgr manager.Manager, c controller.Controller, log logr.Logger, sourceKind string, sourceType client.Object) error {
 	// Setup watches
-	if err := c.Watch(source.Kind(mgr.GetCache(), &corev1.PersistentVolumeClaim{}), handler.EnqueueRequestsFromMapFunc(
-		func(_ context.Context, obj client.Object) []reconcile.Request {
-			pvc := obj.(*corev1.PersistentVolumeClaim)
+	if err := c.Watch(source.Kind(mgr.GetCache(), &corev1.PersistentVolumeClaim{}, handler.TypedEnqueueRequestsFromMapFunc[*corev1.PersistentVolumeClaim](
+		func(_ context.Context, pvc *corev1.PersistentVolumeClaim) []reconcile.Request {
 			if IsPVCDataSourceRefKind(pvc, sourceKind) {
 				pvcKey := types.NamespacedName{Namespace: pvc.Namespace, Name: pvc.Name}
 				return []reconcile.Request{{NamespacedName: pvcKey}}
@@ -133,7 +129,7 @@ func addCommonPopulatorsWatches(mgr manager.Manager, c controller.Controller, lo
 			}
 			return nil
 		}),
-	); err != nil {
+	)); err != nil {
 		return err
 	}
 
@@ -152,9 +148,9 @@ func addCommonPopulatorsWatches(mgr manager.Manager, c controller.Controller, lo
 		return reqs
 	}
 
-	if err := c.Watch(source.Kind(mgr.GetCache(), sourceType),
+	if err := c.Watch(source.Kind(mgr.GetCache(), sourceType,
 		handler.EnqueueRequestsFromMapFunc(mapDataSourceRefToPVC),
-	); err != nil {
+	)); err != nil {
 		return err
 	}
 
@@ -207,7 +203,7 @@ func (r *ReconcilerBase) createPVCPrime(pvc *corev1.PersistentVolumeClaim, sourc
 			Kind:    "PersistentVolumeClaim",
 		}),
 	}
-	cc.SetPvcAllowedAnnotations(pvcPrime, pvc)
+	cc.CopyAllowedAnnotations(pvc, pvcPrime)
 	util.SetRecommendedLabels(pvcPrime, r.installerLabels, "cdi-controller")
 
 	// We use the populator-specific pvcModifierFunc to add required annotations
@@ -253,19 +249,15 @@ func (r *ReconcilerBase) updatePVCWithPVCPrimeAnnotations(pvc, pvcPrime *corev1.
 	return pvcCopy, nil
 }
 
-func (r *ReconcilerBase) updatePVCWithPVCPrimeLabels(pvc *corev1.PersistentVolumeClaim, pvcPrimeLabels map[string]string) error {
+func (r *ReconcilerBase) updatePVCWithPVCPrimeLabels(pvc *corev1.PersistentVolumeClaim, pvcPrimeLabels map[string]string) (*corev1.PersistentVolumeClaim, error) {
 	pvcCopy := pvc.DeepCopy()
-	for label, value := range pvcPrimeLabels {
-		if _, found := pvcCopy.GetLabels()[label]; !found && validLabelsMatch.MatchString(label) {
-			cc.AddLabel(pvcCopy, label, value)
-		}
-	}
+	cc.CopyAllowedLabels(pvcPrimeLabels, pvcCopy, false)
 	if !reflect.DeepEqual(pvc.ObjectMeta, pvcCopy.ObjectMeta) {
 		if err := r.client.Update(context.TODO(), pvcCopy); err != nil {
-			return err
+			return nil, err
 		}
 	}
-	return nil
+	return pvcCopy, nil
 }
 
 // reconcile functions
