@@ -25,6 +25,7 @@ import (
 	. "github.com/onsi/gomega"
 
 	snapshotv1 "github.com/kubernetes-csi/external-snapshotter/client/v6/apis/volumesnapshot/v1"
+
 	corev1 "k8s.io/api/core/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	extv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
@@ -35,6 +36,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
+
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
@@ -283,8 +285,8 @@ var _ = Describe("Datavolume controller reconcile loop", func() {
 		Expect(ok).To(BeFalse())
 	},
 		Entry("with pod running phase", string(corev1.PodRunning)),
-		Entry("with pod succeded phase", string(corev1.PodFailed)),
-		Entry("with pod succeded phase", string(corev1.PodSucceeded)),
+		Entry("with pod succeeded phase", string(corev1.PodFailed)),
+		Entry("with pod succeeded phase", string(corev1.PodSucceeded)),
 	)
 
 	DescribeTable("Should create PVC Prime with proper upload annotations", func(key, value, expectedValue string) {
@@ -346,9 +348,9 @@ func newUploadPopulatorPVC(name string) *corev1.PersistentVolumeClaim {
 		Spec: corev1.PersistentVolumeClaimSpec{
 			StorageClassName: &scName,
 			AccessModes:      []corev1.PersistentVolumeAccessMode{corev1.ReadWriteOnce},
-			Resources: corev1.ResourceRequirements{
+			Resources: corev1.VolumeResourceRequirements{
 				Requests: corev1.ResourceList{
-					corev1.ResourceName(corev1.ResourceStorage): resource.MustParse("1Gi"),
+					corev1.ResourceStorage: resource.MustParse("1Gi"),
 				},
 			},
 			DataSourceRef: &corev1.TypedObjectReference{
