@@ -19,18 +19,11 @@ package controller
 import (
 	"context"
 	"strings"
-
-	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
-	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
+	"time"
 
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-	"sigs.k8s.io/controller-runtime/pkg/client"
-	"sigs.k8s.io/controller-runtime/pkg/client/fake"
-	logf "sigs.k8s.io/controller-runtime/pkg/log"
-	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	ocpconfigv1 "github.com/openshift/api/config/v1"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -39,10 +32,15 @@ import (
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
 
+	"sigs.k8s.io/controller-runtime/pkg/client"
+	"sigs.k8s.io/controller-runtime/pkg/client/fake"
+	logf "sigs.k8s.io/controller-runtime/pkg/log"
+	"sigs.k8s.io/controller-runtime/pkg/reconcile"
+
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
-
-	//cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
+	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
+	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
 	"kubevirt.io/containerized-data-importer/pkg/util/cert/fetcher"
 	"kubevirt.io/containerized-data-importer/pkg/util/naming"
 )
@@ -177,6 +175,46 @@ var _ = Describe("Upload controller reconcile loop", func() {
 
 	})
 
+	It("Should return nil and delete pod if deadline exceeded", func() {
+		testPvc := cc.CreatePvc("testPvc1", "default", map[string]string{cc.AnnUploadRequest: "", cc.AnnPodPhase: string(corev1.PodRunning)}, nil)
+		pod := createUploadPod(testPvc)
+		pod.Status.ContainerStatuses = []corev1.ContainerStatus{
+			{
+				State: corev1.ContainerState{
+					Terminated: &corev1.ContainerStateTerminated{
+						ExitCode: 0,
+						Message:  `{"deadlinePassed": true}`,
+					},
+				},
+			},
+		}
+		reconciler := createUploadReconciler(testPvc,
+			pod,
+			createUploadService(testPvc),
+		)
+
+		_, err := reconciler.Reconcile(context.TODO(), reconcile.Request{NamespacedName: types.NamespacedName{Name: "testPvc1", Namespace: "default"}})
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verifying the pod no longer exists")
+		podList := &corev1.PodList{}
+		err = reconciler.client.List(context.TODO(), podList, &client.ListOptions{})
+		Expect(err).ToNot(HaveOccurred())
+		Expect(podList.Items).To(BeEmpty())
+
+		By("Verifying the service exists")
+		uploadService := &corev1.Service{}
+		err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: createUploadResourceName("testPvc1"), Namespace: "default"}, uploadService)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(uploadService.Name).To(Equal(createUploadResourceName(testPvc.Name)))
+
+		pvc := &corev1.PersistentVolumeClaim{}
+		err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "testPvc1", Namespace: "default"}, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(pvc.Annotations[cc.AnnPodPhase]).To(Equal(""))
+		Expect(pvc.Annotations[cc.AnnPodReady]).To(Equal("false"))
+	})
+
 	It("Should return nil and remove any service and pod if pvc marked for deletion", func() {
 		testPvc := cc.CreatePvc("testPvc1", "default", map[string]string{cc.AnnUploadRequest: "", cc.AnnPodPhase: string(corev1.PodPending)}, nil)
 		testPvc.Finalizers = append(testPvc.Finalizers, "keepmearound")
@@ -283,6 +321,16 @@ var _ = Describe("reconcilePVC loop", func() {
 	testPvcName := "testPvc1"
 	uploadResourceName := "uploader" //createUploadResourceName(testPvcName)
 
+	expectDeadline := func(pod *corev1.Pod) {
+		Expect(pod.Spec.Containers).To(HaveLen(1))
+		for _, env := range pod.Spec.Containers[0].Env {
+			if env.Name == "DEADLINE" {
+				return
+			}
+		}
+		Fail("DEADLINE env var not found")
+	}
+
 	Context("Is clone", func() {
 		isClone := true
 
@@ -336,6 +384,7 @@ var _ = Describe("reconcilePVC loop", func() {
 			Expect(uploadPod.GetAnnotations()[cc.AnnPodNetwork]).To(Equal("net1"))
 			Expect(uploadPod.GetAnnotations()[cc.AnnPodSidecarInjectionIstio]).To(Equal(cc.AnnPodSidecarInjectionIstioDefault))
 			Expect(uploadPod.GetAnnotations()[cc.AnnPodSidecarInjectionLinkerd]).To(Equal(cc.AnnPodSidecarInjectionLinkerdDefault))
+			expectDeadline(uploadPod)
 
 			uploadService = &corev1.Service{}
 			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: naming.GetServiceNameFromResourceName(uploadResourceName), Namespace: "default"}, uploadService)
@@ -433,6 +482,7 @@ var _ = Describe("reconcilePVC loop", func() {
 			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: uploadResourceName, Namespace: "default"}, uploadPod)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(uploadPod.Name).To(Equal(uploadResourceName))
+			expectDeadline(uploadPod)
 
 			uploadService = &corev1.Service{}
 			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: uploadResourceName, Namespace: "default"}, uploadService)
@@ -448,13 +498,13 @@ var _ = Describe("reconcilePVC loop", func() {
 			Expect(err).ToNot(HaveOccurred())
 		})
 
-		DescribeTable("should pass correct crypto config to created pod", func(profile *ocpconfigv1.TLSSecurityProfile) {
+		DescribeTable("should pass correct crypto config to created pod", func(profile *cdiv1.TLSSecurityProfile) {
 			testPvc := cc.CreatePvc(testPvcName, "default", map[string]string{cc.AnnUploadRequest: "", AnnUploadPod: uploadResourceName}, nil)
 			reconciler := createUploadReconciler(testPvc)
 			cdiConfig := &cdiv1.CDIConfig{}
 			err := reconciler.client.Get(context.TODO(), types.NamespacedName{Name: common.ConfigName}, cdiConfig)
 			Expect(err).ToNot(HaveOccurred())
-			profileType := ocpconfigv1.TLSProfileIntermediateType
+			profileType := cdiv1.TLSProfileIntermediateType
 			if profile != nil {
 				profileType = profile.Type
 				cdiConfig.Spec.TLSSecurityProfile = profile
@@ -473,7 +523,7 @@ var _ = Describe("reconcilePVC loop", func() {
 			foundMinVersionEnvVar := false
 			for _, envVar := range uploadPod.Spec.Containers[0].Env {
 				if envVar.Name == common.MinVersionTLSVar {
-					Expect(envVar.Value).To(Equal(string(ocpconfigv1.TLSProfiles[profileType].MinTLSVersion)))
+					Expect(envVar.Value).To(Equal(string(cdiv1.TLSProfiles[profileType].MinTLSVersion)))
 					foundMinVersionEnvVar = true
 				}
 			}
@@ -481,14 +531,46 @@ var _ = Describe("reconcilePVC loop", func() {
 			foundCiphersEnvVar := false
 			for _, envVar := range uploadPod.Spec.Containers[0].Env {
 				if envVar.Name == common.CiphersTLSVar {
-					Expect(envVar.Value).To(Equal(strings.Join(ocpconfigv1.TLSProfiles[profileType].Ciphers, ",")))
+					Expect(envVar.Value).To(Equal(strings.Join(cdiv1.TLSProfiles[profileType].Ciphers, ",")))
 					foundCiphersEnvVar = true
 				}
 			}
 			Expect(foundCiphersEnvVar).To(BeTrue())
 		},
 			Entry("no profile set", nil),
-			Entry("'Old' profile set", &ocpconfigv1.TLSSecurityProfile{Type: ocpconfigv1.TLSProfileOldType, Old: &ocpconfigv1.OldTLSProfile{}}),
+			Entry("'Old' profile set", &cdiv1.TLSSecurityProfile{Type: cdiv1.TLSProfileOldType, Old: &cdiv1.OldTLSProfile{}}),
+		)
+
+		DescribeTable("Should use proper cert duration", func(expectedDuration time.Duration, setCertConfig bool) {
+			testPvc := cc.CreatePvc(testPvcName, "default", map[string]string{cc.AnnUploadRequest: "", AnnUploadPod: uploadResourceName}, nil)
+			reconciler := createUploadReconciler(testPvc)
+			reconciler.serverCertGenerator = &fakeCertGenerator{expectedDuration: expectedDuration}
+			if setCertConfig {
+				cdi := &cdiv1.CDI{}
+				err := reconciler.client.Get(context.TODO(), types.NamespacedName{Name: "cdi"}, cdi)
+				Expect(err).ToNot(HaveOccurred())
+				cdi.Spec.CertConfig = &cdiv1.CDICertConfig{
+					Server: &cdiv1.CertConfig{
+						Duration:    &metav1.Duration{Duration: expectedDuration},
+						RenewBefore: &metav1.Duration{Duration: expectedDuration / 2},
+					},
+				}
+				err = reconciler.client.Update(context.Background(), cdi)
+				Expect(err).ToNot(HaveOccurred())
+			}
+
+			_, err := reconciler.reconcilePVC(reconciler.log, testPvc, isClone)
+			Expect(err).ToNot(HaveOccurred())
+
+			secret := &corev1.Secret{}
+			err = reconciler.client.Get(context.TODO(), types.NamespacedName{Name: uploadResourceName, Namespace: "default"}, secret)
+			Expect(err).ToNot(HaveOccurred())
+
+			fcg := reconciler.serverCertGenerator.(*fakeCertGenerator)
+			Expect(fcg.calls).To(Equal(1))
+		},
+			Entry("default cert duration", 24*time.Hour, false),
+			Entry("configured cert duration", 12*time.Hour, true),
 		)
 	})
 })
@@ -595,7 +677,7 @@ var _ = Describe("updateUploadAnnotations", func() {
 
 		pvcCopy := testPvc.DeepCopy()
 
-		updateUploadAnnotations(pvcCopy.Annotations, pod)
+		setAnnotationsFromPodWithPrefix(pvcCopy.Annotations, pod, nil, cc.AnnRunningCondition)
 		Expect(pvcCopy.Annotations[cc.AnnPodRestarts]).To(Equal("1"))
 		Expect(pvcCopy.GetAnnotations()[cc.AnnRunningCondition]).To(Equal("true"))
 		Expect(pvcCopy.GetAnnotations()[cc.AnnRunningConditionMessage]).To(Equal(""))
