@@ -6,22 +6,21 @@ import (
 
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+
 	"github.com/pkg/errors"
-	corev1 "k8s.io/api/core/v1"
+
 	v1 "k8s.io/api/core/v1"
-	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/record"
 	"k8s.io/utils/ptr"
+
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
 
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	. "kubevirt.io/containerized-data-importer/pkg/controller/common"
 	"kubevirt.io/containerized-data-importer/pkg/util/cert"
-
-	"kubevirt.io/controller-lifecycle-operator-sdk/api"
 	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/api"
 )
 
@@ -34,13 +33,13 @@ var _ = Describe("getVolumeMode", func() {
 	pvcVolumeModeFilesystem := CreatePvc("testPVCVolumeModeFS", "default", map[string]string{AnnSource: SourceHTTP}, nil)
 	pvcVolumeModeFilesystemDefault := CreatePvc("testPVCVolumeModeFS", "default", map[string]string{AnnSource: SourceHTTP}, nil)
 
-	DescribeTable("should", func(pvc *corev1.PersistentVolumeClaim, expectedResult corev1.PersistentVolumeMode) {
+	DescribeTable("should", func(pvc *v1.PersistentVolumeClaim, expectedResult v1.PersistentVolumeMode) {
 		result := GetVolumeMode(pvc)
 		Expect(result).To(Equal(expectedResult))
 	},
-		Entry("return block if pvc has block volume mode", pvcVolumeModeBlock, corev1.PersistentVolumeBlock),
-		Entry("return file system if pvc has filesystem mode", pvcVolumeModeFilesystem, corev1.PersistentVolumeFilesystem),
-		Entry("return file system if pvc has no mode defined", pvcVolumeModeFilesystemDefault, corev1.PersistentVolumeFilesystem),
+		Entry("return block if pvc has block volume mode", pvcVolumeModeBlock, v1.PersistentVolumeBlock),
+		Entry("return file system if pvc has filesystem mode", pvcVolumeModeFilesystem, v1.PersistentVolumeFilesystem),
+		Entry("return file system if pvc has no mode defined", pvcVolumeModeFilesystemDefault, v1.PersistentVolumeFilesystem),
 	)
 })
 
@@ -48,7 +47,7 @@ var _ = Describe("CheckIfLabelExists", func() {
 	pvc := CreatePvc("testPVC", "default", nil, map[string]string{common.CDILabelKey: common.CDILabelValue})
 	pvcNoLbl := CreatePvc("testPVC2", "default", nil, nil)
 
-	DescribeTable("should", func(pvc *corev1.PersistentVolumeClaim, key, value string, expectedResult bool) {
+	DescribeTable("should", func(pvc *v1.PersistentVolumeClaim, key, value string, expectedResult bool) {
 		result := checkIfLabelExists(pvc, key, value)
 		Expect(result).To(Equal(expectedResult))
 	},
@@ -230,6 +229,55 @@ var _ = Describe("setAnnotationsFromPod", func() {
 		Expect(result[AnnRunningConditionReason]).To(Equal(ScratchSpaceRequiredReason))
 		Expect(result[AnnRequiresScratch]).To(Equal("true"))
 	})
+
+	It("Should set image pull failure message and reason", func() {
+		const errorIncludesImagePullText = `Unable to process data: ` + common.ImagePullFailureText + `: reading manifest wrong
+in quay.io/myproject/myimage: manifest unknown`
+
+		result := make(map[string]string)
+		testPod := CreateImporterTestPod(CreatePvc("test", metav1.NamespaceDefault, nil, nil), "test", nil)
+		testPod.Status = v1.PodStatus{
+			ContainerStatuses: []v1.ContainerStatus{
+				{
+					State: v1.ContainerState{
+						Terminated: &v1.ContainerStateTerminated{
+							Message: errorIncludesImagePullText,
+							Reason:  common.GenericError,
+						},
+					},
+				},
+			},
+		}
+		setAnnotationsFromPodWithPrefix(result, testPod, nil, AnnRunningCondition)
+		Expect(result[AnnRunningCondition]).To(Equal("false"))
+		Expect(result[AnnRunningConditionMessage]).To(Equal(errorIncludesImagePullText))
+		Expect(result[AnnRunningConditionReason]).To(Equal(ImagePullFailedReason))
+		Expect(result[AnnRequiresScratch]).To(BeEmpty())
+	})
+
+	It("Should set running reason as error for general errors", func() {
+		const errorMessage = `just a fake error text to check in this test`
+
+		result := make(map[string]string)
+		testPod := CreateImporterTestPod(CreatePvc("test", metav1.NamespaceDefault, nil, nil), "test", nil)
+		testPod.Status = v1.PodStatus{
+			ContainerStatuses: []v1.ContainerStatus{
+				{
+					State: v1.ContainerState{
+						Terminated: &v1.ContainerStateTerminated{
+							Message: errorMessage,
+							Reason:  common.GenericError,
+						},
+					},
+				},
+			},
+		}
+		setAnnotationsFromPodWithPrefix(result, testPod, nil, AnnRunningCondition)
+		Expect(result[AnnRunningCondition]).To(Equal("false"))
+		Expect(result[AnnRunningConditionMessage]).To(Equal(errorMessage))
+		Expect(result[AnnRunningConditionReason]).To(Equal(common.GenericError))
+		Expect(result[AnnRequiresScratch]).To(BeEmpty())
+	})
 })
 
 var _ = Describe("addLabelsFromTerminationMessage", func() {
@@ -326,97 +374,6 @@ var _ = Describe("GetPreallocation", func() {
 	})
 })
 
-var _ = Describe("ValidateClone", func() {
-	sourcePvc := CreatePvc("testPVC", "default", map[string]string{}, nil)
-	blockVM := corev1.PersistentVolumeBlock
-	fsVM := corev1.PersistentVolumeFilesystem
-
-	It("Should reject the clone if source and target have different content types", func() {
-		sourcePvc.Annotations[AnnContentType] = string(cdiv1.DataVolumeKubeVirt)
-		dvSpec := &cdiv1.DataVolumeSpec{ContentType: cdiv1.DataVolumeArchive}
-
-		err := ValidateClone(sourcePvc, dvSpec)
-		Expect(err).To(HaveOccurred())
-		Expect(err.Error()).To(ContainSubstring(
-			fmt.Sprintf("Source contentType (%s) and target contentType (%s) do not match", cdiv1.DataVolumeKubeVirt, cdiv1.DataVolumeArchive)))
-	})
-
-	It("Should reject the clone if the target has an incompatible size and the source PVC is using block volumeMode (Storage API)", func() {
-		sourcePvc.Annotations[AnnContentType] = string(cdiv1.DataVolumeKubeVirt)
-		sourcePvc.Spec.VolumeMode = &blockVM
-		storageSpec := &cdiv1.StorageSpec{
-			Resources: corev1.ResourceRequirements{
-				Requests: corev1.ResourceList{
-					corev1.ResourceStorage: resource.MustParse("1Mi"), // Less than the source's one (1Gi)
-				},
-			},
-		}
-		dvSpec := &cdiv1.DataVolumeSpec{Storage: storageSpec}
-
-		err := ValidateClone(sourcePvc, dvSpec)
-		Expect(err).To(HaveOccurred())
-		Expect(err.Error()).To(ContainSubstring("target resources requests storage size is smaller than the source"))
-	})
-
-	It("Should validate the clone when source PVC is using fs volumeMode, even if the target has an incompatible size (Storage API)", func() {
-		sourcePvc.Annotations[AnnContentType] = string(cdiv1.DataVolumeKubeVirt)
-		sourcePvc.Spec.VolumeMode = &fsVM
-		storageSpec := &cdiv1.StorageSpec{
-			Resources: corev1.ResourceRequirements{
-				Requests: corev1.ResourceList{
-					corev1.ResourceStorage: resource.MustParse("1Mi"), // Less than the source's one (1Gi)
-				},
-			},
-		}
-		dvSpec := &cdiv1.DataVolumeSpec{Storage: storageSpec}
-
-		err := ValidateClone(sourcePvc, dvSpec)
-		Expect(err).ToNot(HaveOccurred())
-	})
-
-	It("Should validate the clone if target's size is empty, even when the source uses block volumeMode (Storage API)", func() {
-		sourcePvc.Annotations[AnnContentType] = string(cdiv1.DataVolumeKubeVirt)
-		sourcePvc.Spec.VolumeMode = &blockVM
-		storageSpec := &cdiv1.StorageSpec{}
-		dvSpec := &cdiv1.DataVolumeSpec{Storage: storageSpec}
-
-		err := ValidateClone(sourcePvc, dvSpec)
-		Expect(err).ToNot(HaveOccurred())
-	})
-
-	It("Should reject the clone when the target has an incompatible size (PVC API)", func() {
-		sourcePvc.Annotations[AnnContentType] = string(cdiv1.DataVolumeKubeVirt)
-		pvcSpec := &corev1.PersistentVolumeClaimSpec{
-			Resources: corev1.ResourceRequirements{
-				Requests: corev1.ResourceList{
-					corev1.ResourceStorage: resource.MustParse("1Mi"), // Less than the source's one (1Gi)
-				},
-			},
-		}
-		dvSpec := &cdiv1.DataVolumeSpec{PVC: pvcSpec}
-
-		err := ValidateClone(sourcePvc, dvSpec)
-		Expect(err).To(HaveOccurred())
-		Expect(err.Error()).To(ContainSubstring("target resources requests storage size is smaller than the source"))
-
-	})
-
-	It("Should validate the clone when both sizes are compatible (PVC API)", func() {
-		sourcePvc.Annotations[AnnContentType] = string(cdiv1.DataVolumeKubeVirt)
-		pvcSpec := &corev1.PersistentVolumeClaimSpec{
-			Resources: corev1.ResourceRequirements{
-				Requests: corev1.ResourceList{
-					corev1.ResourceStorage: resource.MustParse("1Gi"), // Same as the source's
-				},
-			},
-		}
-		dvSpec := &cdiv1.DataVolumeSpec{PVC: pvcSpec}
-
-		err := ValidateClone(sourcePvc, dvSpec)
-		Expect(err).ToNot(HaveOccurred())
-	})
-})
-
 var _ = Describe("HandleFailedPod", func() {
 	pvc := CreatePvc("test-pvc", "test-ns", nil, nil)
 	podName := "test-pod"
@@ -460,7 +417,7 @@ var _ = Describe("check PVC", func() {
 	pvcWithEndPointAnno := CreatePvc("testPvcWithEndPointAnno", "default", map[string]string{AnnEndpoint: "http://test"}, nil)
 	pvcWithCloneRequestAnno := CreatePvc("testPvcWithCloneRequestAnno", "default", map[string]string{AnnCloneRequest: "source-ns/golden-pvc"}, nil)
 
-	DescribeTable("should", func(pvc *corev1.PersistentVolumeClaim, annotation string, expectedResult bool) {
+	DescribeTable("should", func(pvc *v1.PersistentVolumeClaim, annotation string, expectedResult bool) {
 		result := checkPVC(pvc, annotation, utilLog)
 		Expect(result).To(Equal(expectedResult))
 	},
@@ -478,7 +435,7 @@ func createDataVolumeWithStorageClass(name, ns, storageClassName string) *cdiv1.
 		},
 		Spec: cdiv1.DataVolumeSpec{
 			Source: &cdiv1.DataVolumeSource{},
-			PVC: &corev1.PersistentVolumeClaimSpec{
+			PVC: &v1.PersistentVolumeClaimSpec{
 				StorageClassName: &storageClassName,
 			},
 		},
@@ -507,7 +464,7 @@ func createDataVolumeWithStorageClassPreallocation(name, ns, storageClassName st
 		Spec: cdiv1.DataVolumeSpec{
 			Source:        &cdiv1.DataVolumeSource{},
 			Preallocation: &preallocation,
-			PVC: &corev1.PersistentVolumeClaimSpec{
+			PVC: &v1.PersistentVolumeClaimSpec{
 				StorageClassName: &storageClassName,
 			},
 		},
@@ -563,7 +520,7 @@ func createCDIWithWorkload(name, uid string) *cdiv1.CDI {
 			UID:  types.UID(uid),
 		},
 		Spec: cdiv1.CDISpec{
-			Workloads: api.NodePlacement{},
+			Workloads: sdkapi.NodePlacement{},
 		},
 		Status: cdiv1.CDIStatus{
 			Status: sdkapi.Status{
