@@ -115,7 +115,11 @@ func NewHTTPDataSource(endpoint, accessKey, secKey, certDir string, contentType
 		brokenForQemuImg: brokenForQemuImg,
 		contentLength:    contentLength,
 	}
-	httpSource.n = createNbdkitCurl(nbdkitPid, accessKey, secKey, certDir, nbdkitSocket, extraHeaders, secretExtraHeaders)
+	httpSource.n, err = createNbdkitCurl(nbdkitPid, accessKey, secKey, certDir, nbdkitSocket, extraHeaders, secretExtraHeaders)
+	if err != nil {
+		cancel()
+		return nil, err
+	}
 	// We know this is a counting reader, so no need to check.
 	countingReader := httpReader.(*util.CountingReader)
 	go httpSource.pollProgress(countingReader, 10*time.Minute, time.Second)
@@ -133,9 +137,6 @@ func (hs *HTTPDataSource) Info() (ProcessingPhase, error) {
 	if hs.contentType == cdiv1.DataVolumeArchive {
 		return ProcessingPhaseTransferDataDir, nil
 	}
-	if !hs.readers.Convert {
-		return ProcessingPhaseTransferDataFile, nil
-	}
 	if pullMethod, _ := util.ParseEnvVar(common.ImporterPullMethod, false); pullMethod == string(cdiv1.RegistryPullNode) {
 		hs.url, _ = url.Parse(fmt.Sprintf("nbd+unix:///?socket=%s", nbdkitSocket))
 		if err = hs.n.StartNbdkit(hs.endpoint.String()); err != nil {
@@ -143,9 +144,6 @@ func (hs *HTTPDataSource) Info() (ProcessingPhase, error) {
 		}
 		return ProcessingPhaseConvert, nil
 	}
-	// removing check for hs.brokenForQemuImg, and always assuming it is true
-	// revert once we are able to get nbdkit 1.35.8, which contains a fix for the
-	// slow download speed.
 	return ProcessingPhaseTransferScratch, nil
 }
 
@@ -160,7 +158,8 @@ func (hs *HTTPDataSource) Transfer(path string) (ProcessingPhase, error) {
 		if err != nil || size <= 0 {
 			return ProcessingPhaseError, ErrInvalidPath
 		}
-		err = util.StreamDataToFile(hs.readers.TopReader(), file)
+		hs.readers.StartProgressUpdate()
+		err = streamDataToFile(hs.readers.TopReader(), file)
 		if err != nil {
 			return ProcessingPhaseError, err
 		}
@@ -183,7 +182,7 @@ func (hs *HTTPDataSource) TransferFile(fileName string) (ProcessingPhase, error)
 		return ProcessingPhaseError, err
 	}
 	hs.readers.StartProgressUpdate()
-	err := util.StreamDataToFile(hs.readers.TopReader(), fileName)
+	err := streamDataToFile(hs.readers.TopReader(), fileName)
 	if err != nil {
 		return ProcessingPhaseError, err
 	}
@@ -292,7 +291,8 @@ func createHTTPClient(certDir string) (*http.Client, error) {
 	// the default transport contains Proxy configurations to use environment variables and default timeouts
 	transport := http.DefaultTransport.(*http.Transport).Clone()
 	transport.TLSClientConfig = &tls.Config{
-		RootCAs: certPool,
+		RootCAs:    certPool,
+		MinVersion: tls.VersionTLS12,
 	}
 	transport.GetProxyConnectHeader = func(ctx context.Context, proxyURL *url.URL, target string) (http.Header, error) {
 		h := http.Header{}
@@ -336,7 +336,7 @@ func createHTTPReader(ctx context.Context, ep *url.URL, accessKey, secKey, certD
 		brokenForQemuImg = true
 	}
 	// http.NewRequest can only return error on invalid METHOD, or invalid url. Here the METHOD is always GET, and the url is always valid, thus error cannot happen.
-	req, _ := http.NewRequest("GET", ep.String(), nil)
+	req, _ := http.NewRequest(http.MethodGet, ep.String(), nil)
 
 	addExtraheaders(req, allExtraHeaders)
 
@@ -349,9 +349,9 @@ func createHTTPReader(ctx context.Context, ep *url.URL, accessKey, secKey, certD
 	if err != nil {
 		return nil, uint64(0), true, errors.Wrap(err, "HTTP request errored")
 	}
-	if resp.StatusCode != 200 {
-		klog.Errorf("http: expected status code 200, got %d", resp.StatusCode)
-		return nil, uint64(0), true, errors.Errorf("expected status code 200, got %d. Status: %s", resp.StatusCode, resp.Status)
+	if want := http.StatusOK; resp.StatusCode != want {
+		klog.Errorf("http: expected status code %d, got %d", want, resp.StatusCode)
+		return nil, uint64(0), true, errors.Errorf("expected status code %d, got %d. Status: %s", want, resp.StatusCode, resp.Status)
 	}
 
 	if contentType == cdiv1.DataVolumeKubeVirt {
@@ -409,7 +409,7 @@ func (hs *HTTPDataSource) pollProgress(reader *util.CountingReader, idleTime, po
 }
 
 func getContentLength(client *http.Client, ep *url.URL, accessKey, secKey string, extraHeaders []string) (uint64, error) {
-	req, err := http.NewRequest("HEAD", ep.String(), nil)
+	req, err := http.NewRequest(http.MethodHead, ep.String(), nil)
 	if err != nil {
 		return uint64(0), errors.Wrap(err, "could not create HTTP request")
 	}
@@ -425,9 +425,9 @@ func getContentLength(client *http.Client, ep *url.URL, accessKey, secKey string
 		return uint64(0), errors.Wrap(err, "HTTP request errored")
 	}
 
-	if resp.StatusCode != 200 {
-		klog.Errorf("http: expected status code 200, got %d", resp.StatusCode)
-		return uint64(0), errors.Errorf("expected status code 200, got %d. Status: %s", resp.StatusCode, resp.Status)
+	if want := http.StatusOK; resp.StatusCode != want {
+		klog.Errorf("http: expected status code %d, got %d", want, resp.StatusCode)
+		return uint64(0), errors.Errorf("expected status code %d, got %d. Status: %s", want, resp.StatusCode, resp.Status)
 	}
 
 	for k, v := range resp.Header {
@@ -517,7 +517,7 @@ func getExtraHeadersFromSecrets() ([]string, error) {
 }
 
 func getServerInfo(ctx context.Context, infoURL string) (*common.ServerInfo, error) {
-	req, err := http.NewRequestWithContext(ctx, "GET", infoURL, nil)
+	req, err := http.NewRequestWithContext(ctx, http.MethodGet, infoURL, nil)
 	if err != nil {
 		return nil, errors.Wrap(err, "failed to construct request for containerimage-server info")
 	}
