@@ -2,6 +2,7 @@ package operatorcontroller
 
 import (
 	"github.com/machadovilaca/operator-observability/pkg/operatormetrics"
+
 	runtimemetrics "sigs.k8s.io/controller-runtime/pkg/metrics"
 )
 
@@ -12,8 +13,3 @@ func SetupMetrics() error {
 		operatorMetrics,
 	)
 }
-
-// ListMetrics registered prometheus metrics
-func ListMetrics() []operatormetrics.Metric {
-	return operatormetrics.ListMetrics()
-}
