@@ -22,6 +22,7 @@ import (
 	"reflect"
 
 	"github.com/go-logr/logr"
+
 	admissionregistrationv1 "k8s.io/api/admissionregistration/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -33,26 +34,24 @@ import (
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
+
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
 
-	sdk "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk"
-
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
-	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
-	"kubevirt.io/containerized-data-importer/pkg/operator/resources/cluster"
-	"kubevirt.io/containerized-data-importer/pkg/operator/resources/utils"
-
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	cdicontroller "kubevirt.io/containerized-data-importer/pkg/controller"
 	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
+	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
+	"kubevirt.io/containerized-data-importer/pkg/operator/resources/cluster"
+	"kubevirt.io/containerized-data-importer/pkg/operator/resources/utils"
+	sdk "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk"
 	"kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/callbacks"
 )
 
 func addReconcileCallbacks(r *ReconcileCDI) {
 	r.reconciler.AddCallback(&appsv1.Deployment{}, reconcileDeleteControllerDeployment)
 	r.reconciler.AddCallback(&corev1.ServiceAccount{}, reconcileSCC)
-	r.reconciler.AddCallback(&appsv1.Deployment{}, reconcileCreateRoute)
 	r.reconciler.AddCallback(&appsv1.Deployment{}, reconcileCreatePrometheusInfra)
 	r.reconciler.AddCallback(&appsv1.Deployment{}, reconcileRemainingRelationshipLabels)
 	r.reconciler.AddCallback(&appsv1.Deployment{}, reconcileDeleteDeprecatedResources)
@@ -60,6 +59,9 @@ func addReconcileCallbacks(r *ReconcileCDI) {
 	r.reconciler.AddCallback(&appsv1.Deployment{}, reconcilePvcMutatingWebhook)
 	r.reconciler.AddCallback(&extv1.CustomResourceDefinition{}, reconcileSetConfigAuthority)
 	r.reconciler.AddCallback(&extv1.CustomResourceDefinition{}, reconcileHandleOldVersion)
+	if r.haveRoutes {
+		r.reconciler.AddCallback(&appsv1.Deployment{}, reconcileRoute)
+	}
 }
 
 func isControllerDeployment(d *appsv1.Deployment) bool {
@@ -109,7 +111,7 @@ func reconcileDeleteControllerDeployment(args *callbacks.ReconcileCallbackArgs)
 	return nil
 }
 
-func reconcileCreateRoute(args *callbacks.ReconcileCallbackArgs) error {
+func reconcileRoute(args *callbacks.ReconcileCallbackArgs) error {
 	if args.State != callbacks.ReconcileStatePostRead {
 		return nil
 	}
@@ -126,6 +128,10 @@ func reconcileCreateRoute(args *callbacks.ReconcileCallbackArgs) error {
 	}
 	args.Recorder.Event(cr, corev1.EventTypeNormal, createResourceSuccess, "Successfully ensured upload proxy route exists")
 
+	if err := updateUserRoutes(context.TODO(), args.Logger, args.Client, args.Recorder); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -194,6 +200,9 @@ func deleteWorkerResources(l logr.Logger, c client.Client) error {
 		lo := &client.ListOptions{
 			LabelSelector: ls,
 		}
+
+		l.V(1).Info("Deleting worker resources", "type", reflect.TypeOf(lt).Elem().Name())
+
 		if err := cc.BulkDeleteResources(context.TODO(), c, lt, lo); err != nil {
 			return err
 		}
