@@ -25,11 +25,13 @@ import (
 
 	"github.com/kelseyhightower/envconfig"
 	conditionsv1 "github.com/openshift/custom-resource-status/conditions/v1"
+
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/client-go/tools/record"
+
 	"sigs.k8s.io/controller-runtime/pkg/cache"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
@@ -67,6 +69,9 @@ const (
 	deleteResourceFailed   = "DeleteResourceFailed"
 	deleteResourceSuccess  = "DeleteResourceSuccess"
 	dumpInstallStrategyKey = "DUMP_INSTALL_STRATEGY"
+
+	annInjectUploadProxyCert = "operator.cdi.kubevirt.io/injectUploadProxyCert"
+	updateUserRouteSuccess   = "UploadProxyRouteInjectSuccess"
 )
 
 var (
@@ -137,6 +142,11 @@ func newReconciler(mgr manager.Manager) (*ReconcileCDI, error) {
 
 	recorder := mgr.GetEventRecorderFor("operator-controller")
 
+	haveRoutes, err := haveRoutes(uncachedClient)
+	if err != nil {
+		return nil, err
+	}
+
 	r := &ReconcileCDI{
 		client:              restClient,
 		uncachedClient:      uncachedClient,
@@ -147,6 +157,7 @@ func newReconciler(mgr manager.Manager) (*ReconcileCDI, error) {
 		clusterArgs:         clusterArgs,
 		namespacedArgs:      &namespacedArgs,
 		dumpInstallStrategy: dumpInstallStrategy,
+		haveRoutes:          haveRoutes,
 	}
 	callbackDispatcher := callbacks.NewCallbackDispatcher(log, restClient, uncachedClient, scheme, namespace)
 	r.reconciler = sdkr.NewReconciler(r, log, restClient, callbackDispatcher, scheme, mgr.GetCache, createVersionLabel, updateVersionLabel, LastAppliedConfigAnnotation, certPollInterval, finalizerName, false, recorder)
@@ -179,6 +190,7 @@ type ReconcileCDI struct {
 	certManager         CertManager
 	reconciler          *sdkr.Reconciler
 	dumpInstallStrategy bool
+	haveRoutes          bool
 }
 
 // SetController sets the controller dependency
@@ -279,11 +291,21 @@ func (r *ReconcileCDI) getCertificateDefinitions(cdi *cdiv1.CDI) []cdicerts.Cert
 
 		if cdi.Spec.CertConfig.Server != nil {
 			if cdi.Spec.CertConfig.Server.Duration != nil {
-				args.TargetDuration = &cdi.Spec.CertConfig.Server.Duration.Duration
+				args.ServerDuration = &cdi.Spec.CertConfig.Server.Duration.Duration
 			}
 
 			if cdi.Spec.CertConfig.Server.RenewBefore != nil {
-				args.TargetRenewBefore = &cdi.Spec.CertConfig.Server.RenewBefore.Duration
+				args.ServerRenewBefore = &cdi.Spec.CertConfig.Server.RenewBefore.Duration
+			}
+		}
+
+		if cdi.Spec.CertConfig.Client != nil {
+			if cdi.Spec.CertConfig.Client.Duration != nil {
+				args.ClientDuration = &cdi.Spec.CertConfig.Client.Duration.Duration
+			}
+
+			if cdi.Spec.CertConfig.Client.RenewBefore != nil {
+				args.ClientRenewBefore = &cdi.Spec.CertConfig.Client.RenewBefore.Duration
 			}
 		}
 	}
