@@ -22,11 +22,13 @@ import (
 
 	"github.com/go-logr/logr"
 	secv1 "github.com/openshift/api/security/v1"
+
 	corev1 "k8s.io/api/core/v1"
 	apiequality "k8s.io/apimachinery/pkg/api/equality"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
@@ -136,14 +138,13 @@ func (r *ReconcileCDI) watchSecurityContextConstraints() error {
 		Limit: 1,
 	})
 	if err == nil {
-		return r.controller.Watch(source.Kind(r.getCache(), &secv1.SecurityContextConstraints{}), enqueueCDI(r.client))
+		var scc client.Object = &secv1.SecurityContextConstraints{}
+		return r.controller.Watch(source.Kind(r.getCache(), scc, enqueueCDI(r.client)))
 	}
 	if meta.IsNoMatchError(err) {
 		log.Info("Not watching SecurityContextConstraints")
 		return nil
 	}
 
-	log.Info("GOODBYE SCC")
-
 	return err
 }
