@@ -5,20 +5,23 @@ import (
 	"fmt"
 	"time"
 
-	cdiclientset "kubevirt.io/containerized-data-importer/pkg/client/clientset/versioned"
-	"sigs.k8s.io/controller-runtime/pkg/client"
-
-	snapshotv1 "github.com/kubernetes-csi/external-snapshotter/client/v6/apis/volumesnapshot/v1"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 
+	snapshotv1 "github.com/kubernetes-csi/external-snapshotter/client/v6/apis/volumesnapshot/v1"
+
 	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
+
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
 
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
+	cdiclientset "kubevirt.io/containerized-data-importer/pkg/client/clientset/versioned"
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	"kubevirt.io/containerized-data-importer/pkg/controller"
 	cc "kubevirt.io/containerized-data-importer/pkg/controller/common"
@@ -33,6 +36,8 @@ const (
 	importsToKeep         = 1
 	emptySchedule         = ""
 	errorDigest           = "sha256:12345678900987654321"
+	testKubevirtIoKey     = "test.kubevirt.io/test"
+	testKubevirtIoValue   = "testvalue"
 )
 
 var _ = Describe("DataImportCron", Serial, func() {
@@ -72,6 +77,23 @@ var _ = Describe("DataImportCron", Serial, func() {
 
 		By("[AfterEach] Restore the profile")
 		Expect(utils.UpdateStorageProfile(f.CrClient, scName, *originalProfileSpec)).Should(Succeed())
+
+		// Clean up existing dataimportcrons in the environment that we might have switched
+		l, err := labels.Parse(common.DataImportCronLabel)
+		Expect(err).ToNot(HaveOccurred())
+		snapshots := &snapshotv1.VolumeSnapshotList{}
+		err = f.CrClient.List(context.TODO(), snapshots, &client.ListOptions{Namespace: metav1.NamespaceAll, LabelSelector: l})
+		Expect(err).To(Or(
+			Not(HaveOccurred()),
+			Satisfy(meta.IsNoMatchError),
+		))
+		for i := range snapshots.Items {
+			err = f.CrClient.Delete(context.TODO(), &snapshots.Items[i])
+			Expect(err).To(Or(
+				Not(HaveOccurred()),
+				Satisfy(meta.IsNoMatchError),
+			))
+		}
 	})
 
 	updateDigest := func(digest string) func(cron *cdiv1.DataImportCron) *cdiv1.DataImportCron {
@@ -134,6 +156,7 @@ var _ = Describe("DataImportCron", Serial, func() {
 				},
 			}
 			snapshot = utils.WaitSnapshotReady(f.CrClient, snapshot)
+			Expect(snapshot.Labels).To(HaveKeyWithValue(testKubevirtIoKey, testKubevirtIoValue))
 			deleted, err := utils.WaitPVCDeleted(f.K8sClient, name, ns, 30*time.Second)
 			if err != nil {
 				// work around https://github.com/kubernetes-csi/external-snapshotter/issues/957
@@ -305,16 +328,15 @@ var _ = Describe("DataImportCron", Serial, func() {
 
 			By("Verify DataSource was updated")
 			var dataSource *cdiv1.DataSource
-			Eventually(func() bool {
+			Eventually(func(g Gomega) {
 				dataSource, err = f.CdiClient.CdiV1beta1().DataSources(ns).Get(context.TODO(), cron.Spec.ManagedDataSource, metav1.GetOptions{})
-				if errors.IsNotFound(err) {
-					return false
-				}
-				Expect(err).ToNot(HaveOccurred())
+				g.Expect(err).ToNot(HaveOccurred())
 				readyCond := controller.FindDataSourceConditionByType(dataSource, cdiv1.DataSourceReady)
-				return readyCond != nil && readyCond.Status == corev1.ConditionTrue &&
-					getDataSourceName(format, dataSource) == currentImportDv
-			}, dataImportCronTimeout, pollingInterval).Should(BeTrue(), "DataSource was not updated")
+				g.Expect(readyCond).ToNot(BeNil())
+				g.Expect(readyCond.Status).To(Equal(corev1.ConditionTrue))
+				g.Expect(getDataSourceName(format, dataSource)).To(Equal(currentImportDv))
+				g.Expect(dataSource.Labels).To(HaveKeyWithValue(testKubevirtIoKey, testKubevirtIoValue))
+			}, dataImportCronTimeout, pollingInterval).Should(Succeed(), "DataSource was not updated")
 
 			By("Verify cron was updated")
 			Expect(cron.Status.LastImportedPVC).ToNot(BeNil())
@@ -677,7 +699,7 @@ var _ = Describe("DataImportCron", Serial, func() {
 	})
 
 	Context("Change source format of existing DataImportCron", func() {
-		It("[test_id:10034] Should switch from PVC to snapshot sources once format changes", func() {
+		It("[test_id:10034] Should allow switching back and forth from PVC to snapshot sources", func() {
 			if !f.IsSnapshotStorageClassAvailable() {
 				Skip("Volumesnapshot support needed to test DataImportCron with Volumesnapshot sources")
 			}
@@ -751,6 +773,19 @@ var _ = Describe("DataImportCron", Serial, func() {
 			same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, path, utils.UploadFileMD5, utils.UploadFileSize)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(same).To(BeTrue())
+
+			By("Switch back from snap to PVC source")
+			configureStorageProfileResultingFormat(cdiv1.DataImportCronSourceFormatPvc)
+			currentSource = verifySourceReady(cdiv1.DataImportCronSourceFormatPvc, currentImportDv)
+			dataSource, err = f.CdiClient.CdiV1beta1().DataSources(ns).Get(context.TODO(), cron.Spec.ManagedDataSource, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			expectedSource = cdiv1.DataSourceSource{
+				PVC: &cdiv1.DataVolumeSourcePVC{
+					Name:      currentSource.GetName(),
+					Namespace: currentSource.GetNamespace(),
+				},
+			}
+			Expect(dataSource.Spec.Source).To(Equal(expectedSource))
 		})
 	})
 })
@@ -778,7 +813,6 @@ func getDataVolumeSourceRegistry(f *framework.Framework) (*cdiv1.DataVolumeSourc
 
 func updateDataImportCron(clientSet *cdiclientset.Clientset, namespace string, cronName string,
 	update func(cron *cdiv1.DataImportCron) *cdiv1.DataImportCron) func() error {
-
 	return func() error {
 		cron, err := clientSet.CdiV1beta1().DataImportCrons(namespace).Get(context.TODO(), cronName, metav1.GetOptions{})
 		if err != nil {
